pub(crate) fn opcode_pair_allowed(
previous: Option<Opcode>,
proposed: Opcode,
) -> Result<(), ()>Expand description
Should proposed be allowed as an opcode selector output immediately
following an output of previous?
Some pairs of adjacent Opcodes are rejected at the opcode selector level
without causing an entire instruction generation pass to fail.