pub(crate) fn src_registers_allowed(
available: RegisterSet,
op: Opcode,
) -> RegisterSetExpand description
Figure out the allowed register set for an operation, given what’s available in the schedule.
pub(crate) fn src_registers_allowed(
available: RegisterSet,
op: Opcode,
) -> RegisterSetFigure out the allowed register set for an operation, given what’s available in the schedule.