DstRegisterChecker

Struct DstRegisterChecker 

pub(crate) struct DstRegisterChecker<'v> {
    pass: Pass,
    writer_map: &'v RegisterWriterMap,
    writer_info: RegisterWriter,
    op_is_add_shift: bool,
    disallow_equal: Option<RegisterId>,
}
State information returned by Validator::dst_registers_allowed

Fields

pass: Pass

Is this the original or retry pass?

writer_map: &'v RegisterWriterMap

Reference to a table of RegisterWriter information for each register

writer_info: RegisterWriter

The new RegisterWriter under consideration

op_is_add_shift: bool

Was this Opcode::AddShift?

disallow_equal: Option<RegisterId>

Optionally disallow one matching register, used to implement model::disallow_src_is_dst

Implementations

Source

impl<'v> DstRegisterChecker<'v>

pub(crate) fn check(&self, dst: RegisterId) -> bool

Check a single destination register for usability, using context from Validator::dst_registers_allowed