RegisterFile

Struct RegisterFile 

#[repr(C)]
pub(crate) struct RegisterFile([u64; 8]);
Values for all registers in the HashX machine

Guaranteed to have a repr(C) layout that includes each register in order with no padding and no extra fields. The compiled runtime will produce functions that read or write a RegisterFile directly.

Tuple Fields

0: [u64; 8]

Implementations

Source

impl RegisterFile

pub(crate) fn load(&self, id: RegisterId) -> u64

Load a word from the register file.

pub(crate) fn store(&mut self, id: RegisterId, value: u64)

Store a word into the register file.

pub(crate) fn new(key: SipState, input: u64) -> Self

Initialize a new HashX register file, given a key (derived from the seed) and the user-specified hash input word.

pub(crate) fn digest(&self, key: SipState) -> [u64; 4]

Finalize the state of the register file and generate up to 4 words of output in HashX’s final result format.

This splits the register file into two halves, mixes in the siphash keys again to “remove bias toward 0 caused by multiplications”, and runs one siphash round on each half before recombining them.