struct DataSchedule {
latencies: [Cycle; 8],
}Expand description
Latency tracking for all relevant CPU registers
Fields
latencies: [Cycle; 8]Cycle count at which this register is available, indexed by register ID
Implementations
Sourceimpl DataSchedule
impl DataSchedule
Sourcefn plan_register_write(&mut self, dst: RegisterId, cycle: Cycle)
fn plan_register_write(&mut self, dst: RegisterId, cycle: Cycle)
Plan to finish a register write at the indicated cycle
Sourcefn register_available(&self, reg: RegisterId, cycle: Cycle) -> bool
fn register_available(&self, reg: RegisterId, cycle: Cycle) -> bool
Look up if a register will be available at or before the indicated cycle.
Sourcefn overall_latency(&self) -> Cycle
fn overall_latency(&self) -> Cycle
Return the overall latency, the Cycle at which we expect
every register to reach its latest simulated state.