struct SimpleBitArray<const N: usize> {
inner: [usize; N],
}Expand description
Simple packed bit array implementation
This could use the bitvec crate if we cared to depend on it, but the
functionality we need is so tiny let’s keep this simple.
Fields
inner: [usize; N]Array of words to use as a bit vector, in LSB-first order
Implementations