pub(crate) struct SubCycle(u16);Expand description
Sub-cycle timestamp
Timer for instruction decode, at a finer resolution than the Cycle we use for keeping schedule records. Doesn’t need to be compact.
Tuple Fields
0: u16Implementations