pub(crate) struct DstRegisterChecker<'v> {
pass: Pass,
writer_map: &'v RegisterWriterMap,
writer_info: RegisterWriter,
op_is_add_shift: bool,
disallow_equal: Option<RegisterId>,
}Expand description
State information returned by Validator::dst_registers_allowed
Fields
pass: PassIs this the original or retry pass?
writer_map: &'v RegisterWriterMapReference to a table of RegisterWriter information for each register
writer_info: RegisterWriterThe new RegisterWriter under consideration
op_is_add_shift: boolWas this Opcode::AddShift?
disallow_equal: Option<RegisterId>Optionally disallow one matching register, used to implement model::disallow_src_is_dst
Implementations
Sourceimpl<'v> DstRegisterChecker<'v>
impl<'v> DstRegisterChecker<'v>
Sourcepub(crate) fn check(&self, dst: RegisterId) -> bool
pub(crate) fn check(&self, dst: RegisterId) -> bool
Check a single destination register for usability, using context from
Validator::dst_registers_allowed