pub(crate) struct RegisterSet(ArrayVec<RegisterId, NUM_REGISTERS>);Expand description
Identify a set of RegisterIds
This could be done compactly as a u8 bitfield for storage purposes, but in our program generator this is never stored long-term. Instead, we want something the optimizer can reason about as effectively as possible, and we want to optimize for an index() implementation that doesn’t branch. This uses a fixed-capacity array of registers in-set, always sorted.
Tuple Fields
0: ArrayVec<RegisterId, NUM_REGISTERS>Implementations
Sourceimpl RegisterSet
impl RegisterSet
Sourcepub(crate) fn contains(&self, id: RegisterId) -> bool
pub(crate) fn contains(&self, id: RegisterId) -> bool
Test if a register is contained in the set.
Sourcepub(crate) fn from_filter<P: FnMut(RegisterId) -> bool>(predicate: P) -> Self
pub(crate) fn from_filter<P: FnMut(RegisterId) -> bool>(predicate: P) -> Self
Build a new RegisterSet from each register for which a predicate
function returns true.
Sourcepub(crate) fn index(&self, index: usize) -> RegisterId
pub(crate) fn index(&self, index: usize) -> RegisterId
Return a particular register within this set, counting from R0 to R7.
The supplied index must be less than the Self::len() of this set.
Panics if the index is out of range.